@@ -38,7 +38,7 @@
  *
  */
 struct SCP_SESSION*
-scp_session_create();
+scp_session_create(void);
 
 int
 scp_session_set_type(struct SCP_SESSION* s, tui8 type);
