@@ -923,11 +923,6 @@ xrdpvr_write_to_client(void *channel, STREAM *s)
         index += bytes_written;
         bytes_to_send -= bytes_written;
 
-        if ((rv == 0) && (bytes_to_send == 0))
-        {
-            return 0;
-        }
-
         usleep(1000 * 3);
     }
 }
